@@ -22,10 +22,9 @@ from .sym import capitalize_name, tsym, symsymbol, symbol_map, tausym, nusym, oo
 from .dsym import nsym, ksym, zsym
 from .state import state
 from .printing import pprint, pretty, print_str, latex
-from .functions import sqrt, log10, atan2, gcd, exp, Function
+from .functions import sqrt, log10, atan2, gcd, exp, Function, Eq
 from .units import units, u as uu, dB
 from .utils import as_N_D, as_sum
-from .state import state
 import numpy as np
 import sympy as sym
 from sympy.utilities.lambdify import lambdify
@@ -33,7 +32,8 @@ from .sym import simplify
 from .simplify import simplify_sin_cos
 from collections import OrderedDict
 
-__all__ = ('expr', 'symbol', 'symbols', 'deg', 'rad', 'degrees', 'radians')
+__all__ = ('expr', 'symbol', 'symbols', 'deg', 'rad', 'degrees',
+           'radians', 'equation')
 
 class ExprPrint(object):
 
@@ -102,9 +102,16 @@ class ExprPrint(object):
 class ExprContainer(object):    
 
     def evaluate(self):
+        """Evaluate each element to convert to floating point.
+        This may change..."""
         
-        """Evaluate each element to convert to floating point."""        
         return self.__class__([v.evalf() for v in self])
+
+    def evalf(self, n=15):
+        """Evaluate each element to convert to floating point values.
+        `n` is the number of decimal places."""
+
+        return self.__class__([v.evalf(n) for v in self])        
     
     def simplify(self):
         """Simplify each element."""
@@ -819,6 +826,12 @@ class Expr(UndefinedDomain, UndefinedQuantity, ExprPrint, ExprMisc):
         s = '%s(' % name
         print(symdebug(self.expr, s, len(name) + 1))
 
+    @property
+    def sympy(self):
+        """Return SymPy expression."""
+
+        return self.expr
+        
     @property
     def expr_with_units(self):
         """Return SymPy expression with units."""
@@ -1336,7 +1349,8 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         except:
             pass
 
-        if self.var is None:
+        if (self.var is None or self.has(sym.Derivative) or
+            self.has(sym.Integral)):
             self.__ratfun = None
         else:
             # Note, this handles expressions that are products of
@@ -1547,11 +1561,13 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         return self.var not in self.expr.free_symbols
     
     def evaluate(self, arg=None):
-        """Evaluate expression at arg.  arg may be a scalar, or a vector.
+        """Evaluate expression at arg.  `arg` may be a scalar or a vector.
         The result is of type float or complex.
 
+        If arg is iterable, a NumPy array is returned.
+
         There can be only one or fewer undefined variables in the expression.
-        This is replaced by arg and then evaluated to obtain a result.
+        This is replaced by `arg` and then evaluated to obtain a result.
         """
 
         is_causal = self.is_causal
@@ -1578,8 +1594,41 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
                 return 1.0 if abs(arg) <= 0.5 else 0.0
 
             def sinc(arg):
+                """SymPy sinc."""
+
+                # This is used for sinc created by sympify, e.g., a =
+                # expr('sinc(n)').  SymPy uses the unnormalised form
+                # but NumPy (and Lcapy) use the normalised form.
+                # Lambdify does some jiggery pokery and divides the
+                # arg by pi since it is expecting that the NumPy sinc
+                # function is going to be used.  SymPy's choice is
+                # unfortunate from a numerical accuracy point of view
+                # since sincn(n) should be zero for integer n, n != 0.
+
+                # Undo SymPy jiggery pokery.
+                arg = arg * np.pi
+                
                 return 1.0 if arg == 0 else np.sin(np.pi * arg) / (np.pi * arg)
 
+            def sincn(arg):
+                """Normalised sinc."""
+
+                # Note, if sincn is made to print sinc, then lambdify will
+                # call sinc.   Grrrr.
+                
+                return 1.0 if arg == 0 else np.sin(np.pi * arg) / (np.pi * arg)
+
+            def sincu(arg):
+                """Unnormalised sinc."""
+
+                return 1.0 if arg == 0 else np.sin(arg) / arg
+
+            def psinc(M, arg):
+                """Periodic sinc."""
+
+                D = np.sin(np.pi * arg)
+                return 1.0 if D == 0 else np.sin(M * np.pi * arg) / (M * D)
+
             def trap(arg, alpha):
 
                 absarg = abs(arg)
@@ -1638,14 +1687,15 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
             # np.lib.scimath.sqrt converts to complex but cannot be used
             # for lamdification!
             func1 = lambdify(var, expr,
-                            ({'DiracDelta' : dirac,
+                            [{'DiracDelta' : dirac,
                               'Heaviside' : heaviside,
                               'UnitImpulse' : unitimpulse,
                               'UnitStep' : unitstep,
-                              'sinc' : sinc, 'rect' : rect,
-                              'tri' : tri, 'trap' : trap,
+                              'sinc' : sinc, 'sincn' : sincn,
+                              'sincu' : sincu, 'psinc' : psinc,
+                              'rect' : rect, 'tri' : tri, 'trap' : trap,
                               'sqrt' : sqrt, 'exp' : exp},
-                             "scipy", "numpy", "math", "sympy"))
+                             "scipy", "numpy", "math", "sympy"])
 
             def func(arg):
                 # Lambdify barfs on (-1)**n if for negative values of n.
@@ -1852,6 +1902,11 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
     def simplify(self):
         """Simplify expression."""
 
+        # Perhaps roll this into symsimplify?
+        if self.has(AppliedUndef):
+            new, defs = self.remove_undefs(return_mappings=True)
+            return new.simplify().subs(defs)
+
         ret = symsimplify(self.expr)
         return self.__class__(ret, **self.assumptions)
 
@@ -1986,7 +2041,12 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
                               **self.assumptions)
 
     def solve(self, *symbols, **flags):
+        """Solve expression.  This returns a list of solutions."""
 
+        if self.has(AppliedUndef):
+            new, defs = self.remove_undefs(return_mappings=True)
+            return new.solve(*symbols, **flags).subs(defs)
+        
         symbols = [symbol_map(symbol) for symbol in symbols]
         return expr(sym.solve(self.expr, *symbols, **flags))
 
@@ -2069,6 +2129,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         See also general, partfrac, standard, timeconst, and ZPK
 
         """
+        if self.is_Equality:
+            return equation(self.lhs.canonical(), self.rhs.canonical())
+        
         if not self.expr.has(self.var):
             return self
         if self._ratfun is None:
@@ -2083,6 +2146,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
 
         See also canonical, partfrac, standard, timeconst, and ZPK."""
 
+        if self.is_Equality:
+            return equation(self.lhs.general(), self.rhs.general())
+        
         if self._ratfun is None:
             return self.copy()
         return self.__class__(self._ratfun.general(), **self.assumptions)
@@ -2097,6 +2163,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
 
         See also canonical, standard, general, timeconst, and ZPK."""
 
+        if self.is_Equality:
+            return equation(self.lhs.partfrac(), self.rhs.partfrac())
+        
         try:
             if self._ratfun is None:
                 return self.copy()        
@@ -2119,6 +2188,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
 
         See also canonical, standard, general, partfrac, timeconst, and ZPK."""
 
+        if self.is_Equality:
+            return equation(self.lhs.recippartfrac(), self.rhs.recippartfrac())
+        
         if self._ratfun is None:
             return self.copy()
         
@@ -2143,13 +2215,17 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         See also canonical, general, partfrac, timeconst, and ZPK.
 
         """
+
+        if self.is_Equality:
+            return equation(self.lhs.standard(), self.rhs.standard())
+        
         if self._ratfun is None:
             return self.copy()        
         return self.__class__(self._ratfun.standard(), **self.assumptions)
 
     def mixedfrac(self):
         """This is an alias for standard and may be deprecated."""
-        
+
         return self.standard()
 
     def timeconst(self):
@@ -2160,6 +2236,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         See also timeconst_terms, canonical, general, standard,
         partfrac and ZPK."""
 
+        if self.is_Equality:
+            return equation(self.lhs.timeconst(), self.rhs.timeconst())
+        
         if self._ratfun is None:
             return self.copy()        
         return self.__class__(self._ratfun.timeconst(), **self.assumptions)
@@ -2167,6 +2246,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
     def timeconst_terms(self):
         """Convert each term of expression into time constant form."""
 
+        if self.is_Equality:
+            return equation(self.lhs.timeconst_terms(), self.rhs.timeconst_terms())
+        
         result = 0
         for term in self.expr.as_ordered_terms():
             result += self.__class__(term).timeconst()
@@ -2183,7 +2265,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         See also canonical, general, standard, partfrac, and timeconst.
 
         """
-
+        if self.is_Equality:
+            return equation(self.lhs.ZPK(), self.rhs.ZPK())
+        
         if self._ratfun is None:
             return self.copy()        
         return self.__class__(self._ratfun.ZPK(), **self.assumptions)
@@ -2197,6 +2281,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         standard, partfrac, and timeconst.
 
         """
+
+        if self.is_Equality:
+            return equation(self.lhs.factored(), self.rhs.factored())        
         
         if self._ratfun is None:
             return self.copy()
@@ -2210,6 +2297,9 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
 
         See also canonical, general, partfrac, timeconst, and ZPK."""
 
+        if self.is_Equality:
+            return equation(self.lhs.expandcanonoical(), self.rhs.expandcanonical())
+
         if self._ratfun is None:
             return self.copy()        
         return self.__class__(self._ratfun.expandcanonical(), **self.assumptions)
@@ -2499,7 +2589,7 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         return expr(coeffs)
     
     def as_continued_fraction(self):
-        """Convert expression into acontinued fraction."""
+        """Convert expression into a continued fraction."""
 
         def foo(coeffs):
 
@@ -2512,6 +2602,8 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
         return self.__class__(result, **self.assumptions)
 
     def continued_fraction_inverse_coeffs(self):
+        """Convert expression into a continued fraction with inverse
+        coefficients."""
 
         coeffs = []
         var = self.var
@@ -2580,6 +2672,49 @@ As a workaround use x.as_expr() %s y.as_expr()""" % op)
             return self.time()
         return self
 
+    def remove_undefs(self, return_mappings=False):
+        """Replace undefined functions with symbols, for example,
+        replace x(t) with x, etc.
+
+        This is useful for simplifying and solving equations.
+
+        This method gives up if it finds an expression such as
+        x(n - 1) + 2 * x(n) since the arguments are different.
+
+        If return_mappings is True, then a dictionary of substitutions
+        is returned as well as the modified expression.  For example,
+
+        new, defs = expr.remove_undefs(return_mappings=True)
+
+        The original expression can be obtained using:
+
+        new.subs(defs)"""
+        
+        mappings = {}
+        e = self.expr
+        for item in sym.preorder_traversal(e):
+            if isinstance(item, AppliedUndef):
+                name = str(item)
+                parts = name.split('(')
+                name = parts[0]
+                if name in mappings and mappings[name] != item:
+                    # Have found something like x(n) + x(n - 1),
+                    # so give up...   We could create different
+                    # named symbols but these cannot conflict
+                    # with other symbols in the expression.
+                    break
+                
+                mappings[name] = item
+                # Need to propagate complex assumption, etc.
+                e = e.subs(item, expr(name).expr)
+
+        ret = self.__class__(e, **self.assumptions)                
+        if return_mappings:
+            return ret, mappings
+        else:
+            return ret
+
+        
 def exprcontainer(arg, **assumptions):
 
     if isinstance(arg, (ExprList, ExprTuple, ExprDict)):
@@ -2599,7 +2734,7 @@ def exprcontainer(arg, **assumptions):
     raise ValueError('Unsupported exprcontainer %s' % arg.__class__.name)
 
     
-def expr(arg, **assumptions):
+def expr(arg, override=False, **assumptions):
     """Create Lcapy expression from arg.
 
     If `arg` is an `Expr` it is returned, unless `assumptions` is specified.
@@ -2613,6 +2748,9 @@ def expr(arg, **assumptions):
     For example, v = expr('3 * exp(-t / tau) * u(t)')
 
     V = expr('5 * s', causal=True)
+
+    If `override` is True, then create new symbol(s) even if
+    previously defined by SymPy.
     """
 
     from .sym import tsym, fsym, ssym, omegasym
@@ -2628,7 +2766,7 @@ def expr(arg, **assumptions):
     if not isinstance(arg, str) and hasattr(arg, '__iter__'):
         return exprcontainer(arg)
     
-    expr = sympify(arg, **assumptions)
+    expr = sympify(arg, override=override, **assumptions)
 
     symbols = expr.free_symbols
     
@@ -2665,18 +2803,40 @@ def expr_make(domain, arg, **assumptions):
 
     cls = expr_class(domain, arg)
     return cls(arg, **assumptions)
+
+
+def equation(lhs, rhs, **assumptions):
+    """Create an Lcapy equation.
+
+    This is an Lcapy expression of the form Eq(lhs, rhs).
+    For example,
+    e = equation('Y(s)', 'X(s) * 2 * s')
     
+    The left hand side (lhs) and right hand side subexpressions
+    can be obtained with the `lhs` and `rhs` attributes."""
+
+    lhs = expr(lhs)
+    rhs = expr(rhs)
+    # Check if lhs and rhs compatible.
+    diff = lhs - rhs
+
+    cls = lhs.__class__
+    
+    return cls(sym.Eq(lhs.expr, rhs.expr, evaluate=False), **assumptions)
+
 
 def symbol(name, **assumptions):
     """Create an Lcapy symbol.
 
     By default, symbols are assumed to be positive unless real is
     defined or positive is defined as False."""
-    return expr(symsymbol(name, **assumptions))
+    usym = expr(symsymbol(name, **assumptions))
+    state.context.user_symbols[name] = usym
+    return usym
 
 
 def symbols(names, **assumptions):
-    """Create Lcapy symbols from whitespace or comma delimiter string of
+    """Create Lcapy symbols from whitespace or comma delimited string of
     symbol names.  See also symbol."""
 
     from .parser import split
