@@ -53,12 +53,13 @@ class rect(sym.Function):
         return sym.Heaviside(x + S.Half) - sym.Heaviside(x - S.Half)
 
         
-class sinc(sym.Function):
-
+class sincn(sym.Function):
+    
     @classmethod
     def eval(cls, val):
         """
-        Evaluates the sinc (cardinal sine) function.
+        Evaluates the normalised sinc (cardinal sine) function.
+        This is what NumPy uses but not SymPy.
         """
 
         if val.is_Number:
@@ -72,6 +73,46 @@ class sinc(sym.Function):
         x = sym.pi * self.args[0]
         return sym.sin(x) / x
 
+
+class sincu(sym.Function):
+    
+    @classmethod
+    def eval(cls, val):
+        """
+        Evaluates the unnormalised sinc (cardinal sine) function.
+        This is what SymPy uses but not NumPy.
+        """
+
+        if val.is_Number:
+            if val == 0:
+                return S.One
+            return sym.sin(val) / val
+        
+    def rewrite(self, *args, **hints):
+
+        x = self.args[0]
+        return sym.sin(x) / x    
+
+class psinc(sym.Function):
+    
+    @classmethod
+    def eval(cls, M, val):
+        """
+        Evaluates the periodic sinc function.
+        """
+
+        if val.is_Number:
+            if val == 0:
+                return S.One
+            x = sym.pi * val
+            return sym.sin(M * x) / (M * sym.sin(x))
+        
+    def rewrite(self, *args, **hints):
+
+        M = self.args[0]        
+        x = sym.pi * self.args[1]
+        return sym.sin(M * x) / (M * sym.sin(x))
+
     
 class tri(sym.Function):
 
