@@ -2,5 +2,5 @@ class ImpedanceSquaredMixin(object):
 
     quantity = 'impedancesquared'
     quantity_label = 'Impedance^2'
-    units = 'ohm^2'
+    quantity_units = 'ohm^2'
     is_impedancesquared = True
