@@ -31,7 +31,7 @@ from .current import current
 from sympy import Derivative, Integral
 
 
-__all__ = ('V', 'I', 'v', 'i', 'R', 'L', 'C', 'G', 'Y', 'Z',
+__all__ = ('V', 'I', 'v', 'i', 'R', 'NR', 'L', 'C', 'G', 'Y', 'Z',
            'Vdc', 'Vstep', 'Idc', 'Istep', 'Vac', 'sV', 'sI',
            'Iac', 'Vnoise', 'Inoise', 
            'Par', 'Ser', 'Xtal', 'FerriteBead', 'CPE', 'series', 'parallel',
@@ -571,6 +571,7 @@ class Par(ParSer):
     """Parallel class"""
 
     _operator = '|'
+    is_parallel = True
 
     def __init__(self, *args):
 
@@ -695,6 +696,7 @@ class Ser(ParSer):
     """Series class"""
 
     _operator = '+'
+    is_series = True    
 
     def __init__(self, *args):
 
@@ -772,7 +774,8 @@ class Ser(ParSer):
 class R(OnePort):
     """Resistor"""
 
-    is_resistor = True    
+    is_resistor = True
+    is_noiseless = False
     
     def __init__(self, Rval='R'):
 
@@ -787,12 +790,19 @@ class R(OnePort):
     def v_equation(self, i, kind='t'):
 
         return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z).select(kind)
-    
+
+
+class NR(R):
+    """Noiseless resistor"""
+
+    is_noiseless = True
+
 
 class G(OnePort):
     """Conductor"""
 
-    is_conductor = True    
+    is_conductor = True
+    is_noiseless = False    
 
     def __init__(self, Gval='G'):
 
@@ -817,6 +827,12 @@ class G(OnePort):
         return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z).select(kind)
     
 
+class NG(G):
+    """Noiseless conductor"""
+
+    is_noiseless = True
+    
+    
 class L(OnePort):
     """Inductor
 
