@@ -1,10 +1,12 @@
 """This module handles sequences.
 
-Copyright 2020 Michael Hayes, UCECE
+Copyright 2020-2021 Michael Hayes, UCECE
 
 """
 
 from .expr import ExprList, expr
+from .utils import isiterable
+from numpy import array, allclose
 
 # Perhaps subclass numpy ndarray?  But then could not have symbolic
 # elements in the sequence.  Perhaps have flavours for n-domain and
@@ -12,14 +14,35 @@ from .expr import ExprList, expr
 
 class Sequence(ExprList):
 
-    def __init__(self, seq, n=None, evaluate=False, var=None):
+    def __init__(self, seq, ni=None, evaluate=False, var=None):
+        """Sequences can be created from an tuple, list, or ndarray.
+
+        See `seq()` to create a Sequence from a string.
+
+        >>> a = Sequence((1, 2, 3))
+        
+        The sequence indices are specified with the optional `ni` argument.
+        For example:
+
+        >>> a = Sequence((1, 2, 3, 4), (-1, 0, 1, 2))
+
+        If the `ni` argument is not specified, the sequence indices
+        are enumerated from 0.
+
+        The sequence indices can be found using the `n` attribute.
+        This returns a list.
+
+        >>> a = Sequence('{1, _2, 3, 4}').n
+        [-1, 0, 1, 2]
+
+        """
 
         super (Sequence, self).__init__(seq, evaluate)
 
-        if n is None:
-            n = list(range(len(seq)))
+        if ni is None:
+            ni = list(range(len(seq)))
         
-        self.n = n
+        self.n = ni
         self.var = var
 
     @property
@@ -44,7 +67,7 @@ class Sequence(ExprList):
 
         # TODO, support slices, etc.
         try:
-            nindex = self.n.index(n)
+            nindex = list(self.n).index(n)
         except ValueError:
             return expr(0)            
 
@@ -83,8 +106,8 @@ class Sequence(ExprList):
         for m in range(M):
             vals.append(zero)
 
-        n = self.n + list(range(self.n[-1] + 1, len(vals)))
-        return self.__class__(vals, n=n, var=self.var)        
+        ni = self.n + list(range(self.n[-1] + 1, len(vals)))
+        return self.__class__(vals, ni, var=self.var)        
     
     def latex(self):
 
@@ -134,7 +157,20 @@ class Sequence(ExprList):
             items.append(s)
 
         return r'{%s}' % ', '.join(items)
-    
+
+    def as_array(self):
+        """Numerically evaluate and store as NumPy array."""
+
+        # If, for some reason, a sequence can have elements that
+        # depend on n...
+        #vals = array([v1.evaluate(n1) for v1, n1 in zip(self, self.n)])
+        vals = array([v1.cval for v1 in self])
+
+        if allclose(vals.imag, 0.0):
+            vals = vals.real
+        
+        return vals
+        
     def as_impulses(self, var=None):
         """Convert to discrete-time signal in the form of
         a weighted sum of delayed impulses.  For example,
@@ -163,6 +199,33 @@ class Sequence(ExprList):
         
         return result
 
+    def evaluate(self, ni=None):
+        """Evaluate expression at sequence indices specified by `arg`.  `arg`
+        may be a scalar or a vector.  The result is of type float or
+        complex.  Zeroes are returned for indices outside the sequence
+        extent.
+
+        If arg is iterable, a NumPy array is returned.
+
+        """
+        
+        if ni is None:
+            return self.as_array()
+        if isiterable(ni):
+            vals = array([self(n1).cval for n1 in ni])
+
+            if allclose(vals.imag, 0.0):
+                vals = vals.real
+        
+            return vals
+        else:
+            val = self(ni).cval
+
+            if allclose(val.imag, 0.0):
+                val = val.real
+        
+            return val            
+        
     def extent(self):
         """Determine extent of the sequence.
 
@@ -191,11 +254,11 @@ class Sequence(ExprList):
     
         return self.discrete_time_fourier_transform(**assumptions)
 
-    def plot(self, n=None, **kwargs):
-        """Plot the sequence.  If n is not specified, it defaults to the
-        range (-20, 20).  n can be a vector of specified instants, a
-        tuple specifing the range, or a constant specifying the
-        maximum value with the minimum value set to 0.
+    def plot(self, ni=None, **kwargs):
+        """Plot the sequence.  If `ni` is not specified, it defaults to the
+        sequence indices.  `ni` can be a vector of specified sequence
+        indices, a tuple specifing the range, or a constant specifying
+        the maximum value with the minimum value set to 0.
 
         kwargs include:
         axes - the plot axes to use otherwise a new figure is created
@@ -205,9 +268,16 @@ class Sequence(ExprList):
         yscale - the y-axis scaling, say for plotting mV
         in addition to those supported by the matplotlib plot command.
         
-        The plot axes are returned."""
+        The plot axes are returned.
 
-        return self.as_impulses().plot(n, **kwargs)
+        """
+
+        if ni is None:
+            ni = self.n
+
+        # This is not the most efficient way but plot routines expect
+        # an Expr object.
+        return self.as_impulses().plot(ni, **kwargs)
 
     def __call__(self, arg, **assumptions):
         """Convert sequence to n-domain or k-domain expression.
@@ -219,7 +289,10 @@ class Sequence(ExprList):
         if id(arg) in (id(n), id(k)):
             return self.as_impulses(arg)
         if arg in (n, k):
-            return self.as_impulses(arg)        
+            return self.as_impulses(arg)
+
+        # This is experimental and may be deprecated.
+        return self[arg]
 
     def ZT(self, **assumptions):
         return self.as_impulses().ZT(**assumptions)    
@@ -239,7 +312,10 @@ class Sequence(ExprList):
         """Implement digital filter specified by a transfer function.  The
         transfer function is described by a vector `b` of coefficients
         for the numerator and a `a` vector of coefficients for the
-        denominator.
+        denominator. 
+        
+        If you would like the response with initial conditions see
+        `DTfilter.response()`.
 
         For a FIR filter a = [1]."""
 
@@ -270,9 +346,7 @@ class Sequence(ExprList):
                     pass
             y[-1] = yn
                 
-        #n = self.n + list(range(self.n[-1] + 1, len(y)))
-        n = self.n
-        return self.__class__(y, n=n, var=self.var)
+        return self.__class__(y, self.n, var=self.var)
     
     def convolve(self, h, mode='full'):
         """Convolve with h."""
