@@ -31,7 +31,7 @@ class LcapyTester(unittest.TestCase):
     def test_sequence(self):
 
         x = seq((1, 2, 3))
-        e = delta(n) + 2 * delta(n - 1) + + 3 * delta(n - 2)
+        e = delta(n) + 2 * delta(n - 1) + 3 * delta(n - 2)
 
         y = seq((1, 2, 3, 0, 0))
 
@@ -46,7 +46,13 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(h.latex(), r'\left\{\underline{1}, 2\right\}', "latex")                
         self.assertEqual(seq('1, 2, 3'), x, "seq")
         self.assertEqual(seq('{1, _2, 3}').latex(), r'\left\{1, \underline{2}, 3\right\}', "latex")
-        self.assertEqual(e.seq(), x, "seq")        
+        self.assertEqual(e.seq(), x, "seq")
+
+        self.assertEqual(h.n, [0, 1], "seq.n")
+
+        x = seq('{1, _2, 3, 4}')
+        self.assertEqual(x.n, [-1, 0, 1, 2], "seq.n")
+        self.assertEqual(x[0], 2, "seq[0]")        
         
     def test_zexpr(self):
 
@@ -74,3 +80,33 @@ class LcapyTester(unittest.TestCase):
         x2 = exp(n)        
         self.assertEqual(x1, x2, "expr")
         
+    def test_undef_solve(self):
+        e = expr('Eq(y(n), a * y(n - 1) + (1 - a) * x(n))')
+        E = e(z)
+        Y = E.solve('Y')[0]
+
+        Yans = expr('z * (a - 1) * X(z) / (a - z)')
+
+        self.assertEqual(Y, Yans, "solve with undefs")
+        
+    def test_dtfilter(self):
+
+        a = symbol('a')
+        lpf = DTFilter((1 - a, ), (1, -a))
+
+        y = lpf.response(1, [0], (0,2))
+        
+        self.assertEqual(y[0], 1 - a, "IIR impulse response @0")
+        self.assertEqual(y[1], a * (1 - a), "IIR impulse response @1")
+        self.assertEqual(y[2], a**2 * (1 - a), "IIR impulse response @2")
+        
+        y = lpf.response(0, [1], (0,2))
+        
+        self.assertEqual(y[0], a, "transient response @0")
+        self.assertEqual(y[1], a**2, "transient response @1")
+        
+        
+        
+
+        
+        
